<?php

namespace Classes;

if (!defined('_PS_VERSION_')) {
    exit;
}

use \Classes\Helper\Logger as zipMoneyLogger;

/**
 * @class    zipMoney
 * @version  1.0.2
 * @package  zipMoney Payments
 * @author   zipMoney Payments
 */
class zipMoney
{

    protected $_api = null;
    protected $_gateway = null;

    const VERSION  = '1.0.0';
    const PLATFORM  = 'Prestashop';
    const CLIENT = 'Prestashop zipMoney Payment';


    public function __construct()
    {
        $this->context = \Context::getContext();
    }

    /**
     * Bootstrapping function.
     *
     * @access public
     */
    public static function bootstrap()
    {
        zipMoneyLogger::setup();
    }

    /**
     * Creates an order.
     *
     * @access public
     * @param  Cart $cart
     */
    public function validateOrder($cart)
    {
        try {
            $order_state = $this->getOrderStatus("zip_pending");
            $total = $cart->getOrderTotal(true);
            $zipMoneyPayment = new \ZipmoneyPayment;
            $payment_method  = $zipMoneyPayment->name;
            if ($zipMoneyPayment->validateOrder($cart->id, $order_state, $total, $payment_method, null, null, null, null, $cart->secure_key, null)) {
                $current_order_id = $zipMoneyPayment->currentOrder;
                $order = new \Order($current_order_id);
                return $order;
            }
            return false;
        } catch (\Exception $e) {
            zipMoneyLogger::error($e->getMessage());
        }
    }

    /**
     * Prepares config detail for checkout request
     *
     * @access protected
     * @return object $config
     */
    protected function _setConfiguration()
    {
        $config = new \zipMoney\Model\CheckoutConfiguration();
        $zipMoneyPayment = new \ZipmoneyPayment;
        $is_iframe_flow = $zipMoneyPayment->useIframeCheckout();
        $redirectUri = $this->context->link->getModuleLink('zipmoneypayment', 'validation', array());
        if ($is_iframe_flow) {
            $redirectUri = $this->context->link->getModuleLink('zipmoneypayment', 'validation', array('iframe' => true));
        }
        $config->setRedirectUri($redirectUri);
        return $config;
    }

    /**
     * Prepares order detail for refund request
     *
     * @access protected
     * @param  object $order
     * @return object $orderInfo
     */
    protected function _setRefundOrder($order)
    {
        $orderInfo = new \zipMoney\Model\CreateRefundRequest();

        $orderInfo->reason = "Prestashop Refund";
        $orderInfo->amount = (float) $order->total_paid;
        $orderInfo->metadata = array(
            'shipping_value' => $order->total_shipping,
            'tax' => $order->total_paid_tax_incl - $order->total_paid_tax_excl
        );

        return $orderInfo;
    }

    protected function getCartFromOrder($order)
    {
        return \Cart::getCartByOrderId((int) $order->id);
    }

    /**
     * Prepares order detail for checkout request based on the $order object
     *
     * @access protected
     * @param  object $order
     * @return object $orderInfo
     */
    protected function _setItemsFromOrder($order)
    {
        $cart = $this->getCartFromOrder($order);
        return $this->getOrderItems($cart);
    }

    /**
     * Prepares order detail for checkout request
     *
     * @access protected
     * @param  object $cart
     * @return object $orderInfo
     */
    protected function _setOrder($cart)
    {
        $orderInfo = new \zipMoney\Model\CheckoutOrder();

        $orderTotal = $cart->getOrderTotal();

        $orderInfo->setCartReference($cart->id);
        $orderInfo->setReference((string)$cart->id);
        $orderInfo->setAmount((float) $orderTotal);

        $currency = new \Currency($cart->id_currency);
        $orderInfo->setCurrency($currency->iso_code);

        $orderInfo->setItems($this->getOrderItems($cart));

        $orderInfo->setShipping($this->_setShippingInfo($cart));

        return $orderInfo;
    }

    /**
     * Always uses cart object to obtain order items for easier management across prestashop versions.
     *
     * @access protected
     * @param  object $cart
     * @return object $orderInfo
     */
    protected function getOrderItems($cart)
    {
        $orderItems = array();
        if ($items = $cart->getProducts()) {
            foreach ($items as $item) {
                $order_item = new \zipMoney\Model\OrderItem();
                $id = $item['id_product'];
                $order_item->setName($item['name']);
                $order_item->setAmount(round((float)$item['price_wt'], 2));
                $order_item->setReference($id);
                $order_item->setDescription($this->getProductDesc($id));
                $order_item->setQuantity($item['quantity']);
                $order_item->setType('sku');
                $imageUri = $this->getProductImage($id);
                if ($this->isSafeUri($imageUri)) {
                    $order_item->setImageUri($imageUri);
                }
                $order_item->setProductCode($this->getProductSKU($id));

                $orderItems[] = $order_item;
            }
        }

        $cartProtocol = \Cart::ONLY_PRODUCTS_WITHOUT_SHIPPING;
        $shipping_value = (float) $cart->getTotalShippingCost();
        $discount_total = $cart->getOrderTotal(true, \Cart::ONLY_DISCOUNTS);

        if ($shipping_value > 0) {
            $shipping_item = new \zipMoney\Model\OrderItem();
            $shipping_item->setName('Shipping');
            $shipping_item->setAmount(round($shipping_value, 2));
            $shipping_item->setQuantity(1);
            $shipping_item->setType('shipping');
            $orderItems[] = $shipping_item;
        }

        // We need to get an example of a discount object
        if ($discount_total > 0) {
            $discounts_item = new \zipMoney\Model\OrderItem();
            $discounts_item->setName('Discount');
            $discounts_item->setAmount(round($discount_total, 2) * -1);
            $discounts_item->setQuantity(1);
            $discounts_item->setType('discount');
            $orderItems[] = $discounts_item;
        }

        return $orderItems;
    }

    /**
     * Returns product image.
     *
     * @access protected
     * @param  int $id_product
     * @return string $id_product
     */
    public function getProductImage($id_product)
    {

        $image     = \Image::getCover($id_product);
        $product   = new \Product($id_product, false, \Context::getContext()->language->id);
        $link      = new \Link;
        $imagePath = $link->getImageLink($product->link_rewrite, $image['id_image'], 'home_default');

        $imageUri = $this->ensureUriProtocol($imagePath);

        return $imageUri;
    }

    /**
     * Returns product description.
     *
     * @access protected
     * @param  int $id_product
     * @return string $id_product
     */
    public function getProductDesc($id_product)
    {

        $product   = new \Product($id_product, false, \Context::getContext()->language->id);

        return $product->description_short;
    }

    /**
     * Returns product reference/sku.
     *
     * @access protected
     * @param  int $id_product
     * @return string $id_product
     */
    public function getProductSKU($id_product)
    {

        $product   = new \Product($id_product, false, \Context::getContext()->language->id);

        return $product->reference;
    }

    /**
     * Returns product categories.
     *
     * @access public
     * @param  int $id_product
     * @return array()
     */
    public function getProductCategories($id_product)
    {
        $categories = \Product::getProductCategories($id_product);
        if (is_array($categories)) {
            foreach ($categories as $cat_id) {
                $category   = new \Category((int)$cat_id, \Context::getContext()->language->id);
                $cat_name[] = $category->name[1];
            }
        }

        return implode(",", $cat_name);
    }

    /**
     * Sets billing info to the request
     *
     * @access protected
     * @param  object $cart
     * @return object $billing_address
     */
    protected function _setBillingInfo($cart)
    {
        $address = new \Address($cart->id_address_invoice);

        $billingAddress  = new \zipMoney\Model\Address();

        if (empty($address->address1)) {
            return null;
        }

        $billingAddress->setFirstName($address->firstname);
        $billingAddress->setLastName($address->lastname);
        $billingAddress->setLine1($address->address1);
        $billingAddress->setLine2($address->address2);
        $billingAddress->setCity($address->city);
        $billingAddress->setPostalCode($address->postcode);

        $state = new \State($address->id_state);
        $country = new \Country($address->id_country);
        if ($state) {
            $billingAddress->setState($state->iso_code);
        }
        if ($country) {
            $billingAddress->setCountry($country->iso_code);
        }

        return $billingAddress;
    }

    /**
     * Sets shipping info to the request array
     *
     * @access protected
     * @param  object $cart
     * @return object $shippingAddress
     */
    protected function _setShippingInfo($cart)
    {
        $address = new \Address($cart->id_address_delivery);

        $shippingAddress  = new \zipMoney\Model\Address();

        if (empty($address->address1)) {
            return null;
        }

        $shippingAddress->setFirstName($address->firstname);
        $shippingAddress->setLastName($address->lastname);
        $shippingAddress->setLine1($address->address1);
        $shippingAddress->setLine2($address->address2);
        $shippingAddress->setCity($address->city);
        $shippingAddress->setPostalCode($address->postcode);

        $state = new \State($address->id_state);
        $country = new \Country($address->id_country);
        if ($state) {
            $shippingAddress->setState($state->iso_code);
        }
        if ($country) {
            $shippingAddress->setCountry($country->iso_code);
        }

        $shipping = new \zipMoney\Model\OrderShipping();
        $shipping->setAddress($shippingAddress);
        $shipping->setPickup(false);

        return $shipping;
    }

    /**
     * Sets customer info to the request array
     *
     * @access protected
     * @param  object $order
     * @return object $shopper
     */
    protected function _setShopperInfo($cart)
    {

        $shopper = new \zipMoney\Model\Shopper();

        $lifetime_sales_refunded_amount = 0;
        $lifetime_sales_amount = 0;
        $average_sale_value     = 0;
        $maximum_sale_value     = 0;
        $i = 0;

        $customer = new \Customer($cart->id_customer);

        if ($cart) {
            $shopper->setEmail($customer->email);
            $shopper->setFirstName($customer->firstname);
            $shopper->setLastName($customer->lastname);
            if ($customer->birthday) {
                if ($customer->birthday != '0000-00-00') {
                    $shopper->setBirthDate($customer->birthday);
                }
            }
        }

        if ($customer->isLogged()) {
            $customer_orders = \Order::getCustomerOrders($customer->id);
            $lifetime_sales_amount = 0;
            if (is_array($customer_orders)) {
                foreach ($customer_orders as $c_order) {
                    $order_total = $c_order['total_paid'];

                    if ($c_order['current_state'] == 18) {
                        $i++;
                        $lifetime_sales_amount += $order_total;

                        if ($order_total > $maximum_sale_value) {
                            $maximum_sale_value = $order_total;
                        }
                    } elseif ($c_order['current_state'] == 10) {
                        $lifetime_sales_refunded_amount += $order_total;
                    }
                }

                if ($i > 0) {
                    $average_sale_value = (float) round($lifetime_sales_amount / $i, 2);
                }
            }
        }

        if (!$shopper->getEmail()) {
            return false;
        }

        $statistics = new \zipMoney\Model\ShopperStatistics();
        $statistics->setSalesTotalAmount((float) $lifetime_sales_amount);
        $statistics->setSalesTotalCount((int) $i);
        $statistics->setSalesAvgAmount((float) $average_sale_value);
        $statistics->setSalesMaxAmount((float) $maximum_sale_value);
        $statistics->setRefundsTotalAmount((float) $lifetime_sales_refunded_amount);

        $currency = new \Currency($cart->id_currency);
        $statistics->setCurrency($currency->iso_code);

        $shopper->setStatistics($statistics);

        $shopper->setBillingAddress($this->_setBillingInfo($cart));

        return $shopper;
    }

    /**
     * Checks if the capture is set
     *
     * @access public
     * @return boolean
     */
    public function isCapture()
    {
        return \Configuration::get('ZM_CAPTURE_METHOD') == 'capture' ? true : false;
    }

    /**
     * Returns the api environment
     *
     * @access public
     * @return string
     */
    public function getEnvironment()
    {
        return \Configuration::get('ZM_ENVIRONMENT');
    }

    /**
     * Returns the platform and module version
     *
     * @access public
     * @return string
     */
    public function getPlatform()
    {
        $zipMoneyPayment = new \ZipmoneyPayment;
        return "Prestashop/" . _PS_VERSION_ . " " . $zipMoneyPayment->displayName . "/" . $zipMoneyPayment->version;
    }

    /**
     * Returns the merchant private key
     *
     * @access public
     * @return string
     */
    public function getMerchantKey()
    {
        return \Configuration::get('ZM_MERCHANT_PRIVATE_KEY');
    }

    /**
     * Checks and alters a parameter url to include web protocols.
     *
     * @access public
     * @param  String $url
     * @return String $url
     */
    public function ensureUriProtocol($url)
    {
        if (strpos($url, 'http://') !== 0 && strpos($url, 'https://') !== 0) {
            if (strpos($url, '//') === 0) {
                $url = "http:" . $url;
            } else {
                $url = "http://" . $url;
            }
        }
        return $url;
    }

    /**
     * Validates whether the url is actually a url
     *
     * @access public
     * @param string $url
     * @return boolean
     */
    public function isSafeUri($url)
    {
        return filter_var($url, FILTER_VALIDATE_URL) === false ? 0 : 1;
    }

    /**
     * Returns the order state
     *
     * @access public
     * @param  string $template , int $language_id
     * @return string
     */
    public static function getOrderStatus($template, $language_id = 1)
    {
        return \Db::getInstance()->getValue('SELECT id_order_state FROM ' . _DB_PREFIX_ . 'order_state_lang WHERE template = "' . pSQL($template) . '" AND id_lang = "' . $language_id . '"');
    }

    /**
     * Stores zip checkout id against a cart id within the cart table.
     *
     * @access public
     * @param $checkout_id
     * @param $cart_id
     */
    public function storeCheckoutId($checkout_id, $cart_id)
    {
        if (!$checkout_id || !$cart_id) {
            return false;
        }
        return \Db::getInstance()->update(
            'cart',
            array(
                'zip_checkout_id' => $checkout_id
            ),
            '`id_cart` = ' . (int) $cart_id,
            1
        );
    }

    /**
     * Returns cart id from database using the zip checkout id
     *
     * @access public
     * @param $checkout_id
     * @return $cart_id
     */
    public function getCartIdFromCheckoutId($checkout_id)
    {
        if (!$checkout_id) {
            return false;
        }
        $cart_id = \Db::getInstance()->getValue("SELECT id_cart FROM " . _DB_PREFIX_ . "cart WHERE zip_checkout_id = '" . $checkout_id . "'");
        return $cart_id;
    }

    /**
     * Returns the payment modules
     *
     * @access public
     * @return array() or string
     */
    public function getPaymentModules()
    {
        return \Module::getPaymentModules();
    }

    /**
     * @return object $metadata
     */
    protected function _setMetaData()
    {
        // object not working must use array
        $zipMoneyPayment = new \ZipmoneyPayment;
        $metadata['platform'] = 'PrestaShop';
        $metadata['platform_version'] = _PS_VERSION_;
        $metadata['plugin'] = 'zipmoneypayment';
        $metadata['plugin_version'] = $zipMoneyPayment->version;
        return $metadata;
    }
}
