<?php

namespace Classes;

if (!defined('_PS_VERSION_')) {
    exit;
}

use \Classes\Helper\Logger as zipMoneyLogger;
use \Classes\Helper\Sdk;
use zipMoney\ApiException;
use \Classes\Helper\ErrorHandling;
use Classes\Helper\Api as ApiHelper;
use zipMoney\Model\CurrencyUtil;

/**
 * @class    zipMoneyStandard
 * @version  1.0.2
 * @package  zipMoney Payments
 * @author   zipMoney Payments
 */
class zipMoneyStandard extends zipMoney
{

    /**
     * Handles standard checkout. Makes a request to /checkout endpoint.
     * Returns redirect_url when successful.
     *
     * @access public
     * @return mixed boolean | string $redirect_url
     */
    public function handleCheckout($cart)
    {
        $zip   =  ApiHelper::getApi();
        \zipMoney\Configuration::getDefaultConfiguration()->setApiKey('Authorization', "Bearer " . $this->getMerchantKey());
        \zipMoney\Configuration::getDefaultConfiguration()->setEnvironment($this->getEnvironment());
        \zipMoney\Configuration::getDefaultConfiguration()->setPlatform($this->getPlatform());

        $checkoutApi = Sdk::getApi(Sdk::CHECKOUT_API_CLASS);

        $checkoutPayload = $this->_prepareRequestForCheckout($cart);
        zipMoneyLogger::debug("Checkout Request = " . json_encode(json_decode($checkoutPayload)));

        try {
            $idempotency = md5(time());
            $response = $checkoutApi->checkoutsCreate($checkoutPayload, $idempotency);
            zipMoneyLogger::debug("Checkout Response:- " . json_encode(json_decode($response)));
            if ($response) {
                $redirectUri = $response->getUri();
                $currency = new \Currency($cart->id_currency);
                $currencyCode = $currency->iso_code;
                $zipMoneyPayment = new \ZipmoneyPayment;
                $is_iframe_flow = $zipMoneyPayment->useIframeCheckout();
                if ($is_iframe_flow && $currencyCode == CurrencyUtil::CURRENCY_NZD) {
                    $redirectUri = $redirectUri . '&embedded=true';
                }
                return $redirectUri;
            }

            return false;
        } catch (ApiException $e) {
            $error = \Tools::jsonEncode(array('message' => $e->getResponseBody()));
            zipMoneyLogger::error("Error:-" . $e->getCode() . "-" . json_encode($e->getResponseBody()));
        } catch (\Exception $e) {
            zipMoneyLogger::error("Error := " . $e->getMessage());
        }
    }

    /**
     * Makes a request to /checkout endpoint using the handleCheckout() method. Prints json response.
     *
     * @access public
     */
    public function getredirecturl()
    {
        $redirect_url = $this->handleCheckout(null);
        $json = array('redirect_url' => $redirect_url);
        header("Content-type:application/json");
        die(json_encode($json));
    }

    /**
     * Handles order refunds. Makes a request to /refund endpoint. Returns redirect_url when successful.
     *
     * @access public
     * @param  object $order, float $amount, string $reason (optional)
     * @return mixed boolean | zipMoney\Response object
     */
    public function handleRefund($order, $amount, $reason = null)
    {
        \zipMoney\Configuration::getDefaultConfiguration()->setApiKey('Authorization', "Bearer " . $this->getMerchantKey());
        \zipMoney\Configuration::getDefaultConfiguration()->setEnvironment($this->getEnvironment());
        \zipMoney\Configuration::getDefaultConfiguration()->setPlatform($this->getPlatform());

        $refundApi = Sdk::getApi(Sdk::REFUND_API_CLASS);

        $refundPayload = $this->_prepareRequestForRefund($order, $amount, $reason);
        zipMoneyLogger::debug("Refund Request = " . json_encode(json_decode($refundPayload->__toString())));

        try {
            $idempotency = md5(time());
            $response  = $refundApi->refundsCreate($refundPayload, $idempotency);

            zipMoneyLogger::debug("Refund Response:- " . json_encode(json_decode($response->__toString())));

            if ($response) {
                return $response;
            }

            return false;
        } catch (ApiException $e) {
            $error = \Tools::jsonEncode(array('message' => $e->getResponseBody()));
            zipMoneyLogger::error("Error:-" . $e->getCode() . "-" . json_encode($e->getResponseBody()));
            if (
                $e->getCode() == 402 &&
                $mapped_error_code = ErrorHandling::getMappedErrorCode($e->getResponseObject()->getError()->getCode())
            ) {
                if ($mapped_error_code) {
                    $this->context->smarty->assign('ps_error_code', $mapped_error_code);
                    return array('zm_error' => $e->getResponseObject()->getError()->getCode());
                }
            }
            return 'error';
        } catch (\Exception $e) {
            zipMoneyLogger::error("Error := " . $e->getMessage());
        }
    }

    /**
     * Handles charge requests. Makes a request to /charges endpoint. Returns a result when successful.
     *
     * @access public
     * @param  object $order, string $checkout_id
     * @return zipMoney\Response object
     */
    public function handleCharge($order, $checkout_id)
    {
        \zipMoney\Configuration::getDefaultConfiguration()->setApiKey('Authorization', "Bearer " . $this->getMerchantKey());
        \zipMoney\Configuration::getDefaultConfiguration()->setEnvironment($this->getEnvironment());
        \zipMoney\Configuration::getDefaultConfiguration()->setPlatform($this->getPlatform());

        $chargePayload = new \zipMoney\Model\CreateChargeRequest();

        $authority = new \zipMoney\Model\Authority();
        $authority->setType('checkout_id');
        $authority->setValue($checkout_id);

        $chargePayload->setAuthority($authority);
        $chargePayload->setReference($order->reference);
        $chargePayload->setAmount((float) $order->total_paid);

        $currency = new \Currency($order->id_currency);
        $chargePayload->setCurrency($currency->iso_code);

        zipMoneyLogger::debug("Is capture := " . $this->isCapture());

        $chargePayload->setCapture($this->isCapture());

        $chargeOrder = new \zipMoney\Model\ChargeOrder();
        $chargeOrder->setReference($order->reference);
        $chargeOrder->setShipping($this->_setShippingInfo($order));
        $chargeOrder->setItems($this->_setItemsFromOrder($order));
        $chargeOrder->setCartReference($order->id_cart);
        $chargePayload->setOrder($chargeOrder);

        $chargePayload->setMetaData(array("order_reference" => $order->reference));

        zipMoneyLogger::debug("Charge Request := " . json_encode(json_decode($chargePayload->__toString())));

        try {
            $charge = Sdk::getApi(Sdk::CHARGE_API_CLASS);
            $idempotency = md5(time());
            $response = $charge->chargesCreate($chargePayload, $idempotency);
            zipMoneyLogger::debug("Charge Response := " . json_encode(json_decode($response->__toString())));

            return $response;
        } catch (ApiException $e) {
            \Tools::jsonEncode(array('message' => $e->getResponseBody()));
            zipMoneyLogger::error("Error:-" . $e->getCode() . "-" . json_encode($e->getResponseBody()));
            $zmError = "";
            if (
                $e->getCode() == 402 &&
                $mapped_error_code = ErrorHandling::getMappedErrorCode($e->getResponseObject()->getError()->getCode())
            ) {
                if ($mapped_error_code) {
                    $zmError = $mapped_error_code;
                }
            }
            return array('redirect' => $this->context->link->getModuleLink('zipmoneypayment', 'zipmoney_error', array()) . ($zmError ? '&zmError=' . $zmError : ''));
        } catch (\Exception $e) {
            zipMoneyLogger::error("Error := " . $e->getMessage());
        }

        return;
    }

    /**
     * Handles authorised order captures. Makes a request to /charges/{id}/capture endpoint. Returns a capture result when successful.
     *
     * @access public
     * @param  object $order, object $charge
     * @return zipMoney\Response object
     */
    public function handleCapture($order, $charge)
    {
        \zipMoney\Configuration::getDefaultConfiguration()->setApiKey('Authorization', "Bearer " . $this->getMerchantKey());
        \zipMoney\Configuration::getDefaultConfiguration()->setEnvironment($this->getEnvironment());
        \zipMoney\Configuration::getDefaultConfiguration()->setPlatform($this->getPlatform());

        if (!$order || !$charge) {
            return false;
        }
        $charge_id = $charge->getId();
        if (!$charge_id) {
            return false;
        }
        if ($charge->getState() != 'authorised' && $charge->getState() != 'approved') {
            return false;
        }
        $captureAmount = (float) $order->total_paid;
        if ($captureAmount <= 0) {
            zipMoneyLogger::error("Capture amount invalid: " . $captureAmount);
            return false;
        }

        $chargeApi = Sdk::getApi(Sdk::CHARGE_API_CLASS);
        try {
            $capturePayload = new \zipMoney\Model\CaptureChargeRequest();
            $capturePayload->setAmount($captureAmount);
            $idempotency = md5(time());
            zipMoneyLogger::debug("Capture Request := " . json_encode(json_decode($capturePayload->__toString())));
            $result = $chargeApi->chargesCapture($charge_id, $capturePayload, $idempotency);
            zipMoneyLogger::debug("Capture Response := " . json_encode(json_decode($result->__toString())));
            zipMoneyLogger::info(sprintf("zipMoney Capture Success [Order Id %s] - From Status Change. Status changed to processing. ", $order->id));
            return $result;
        } catch (ApiException $e) {
            $error = \Tools::jsonEncode(array('message' => $e->getResponseBody()));
            zipMoneyLogger::error("Error:-" . $e->getCode() . "-" . json_encode($e->getResponseBody()));
            if (
                $e->getCode() == 402 &&
                $mapped_error_code = ErrorHandling::getMappedErrorCode($e->getResponseObject()->getError()->getCode())
            ) {
                if ($mapped_error_code) {
                    $this->context->smarty->assign('ps_error_code', $mapped_error_code);
                    return array('zm_error' => $mapped_error_code);
                }
            }
            return 'error';
        } catch (\Exception $e) {
            $error = \Tools::jsonEncode(array('message' => $e->getMessage()));
            zipMoneyLogger::error("Payment Exception: " . $error);
        }
    }

    /**
     * Handles authorised order cancellation. Makes a request to /charges/{id}/cancel endpoint. Returns a cancel result when successful.
     *
     * @access public
     * @param  object $order, object $charge
     * @return zipMoney\Response object
     */
    public function handleCancel($order, $charge)
    {
        \zipMoney\Configuration::getDefaultConfiguration()->setApiKey('Authorization', "Bearer " . $this->getMerchantKey());
        \zipMoney\Configuration::getDefaultConfiguration()->setEnvironment($this->getEnvironment());
        \zipMoney\Configuration::getDefaultConfiguration()->setPlatform($this->getPlatform());

        if (!$order || !$charge) {
            return false;
        }
        $charge_id = $charge->getId();
        if (!$charge_id) {
            return false;
        }
        if ($charge->getState() != 'authorised') {
            return false;
        }

        $chargeApi = Sdk::getApi(Sdk::CHARGE_API_CLASS);
        try {
            $idempotency = md5(time());
            $result = $chargeApi->chargesCancel($charge_id, $idempotency);
            zipMoneyLogger::debug("Cancel Charge Response := " . json_encode(json_decode($result->__toString())));
            zipMoneyLogger::info(sprintf("zipMoney Cancelled [Order Id %s] - From Status Change. Status changed to Cancelled. ", $order->id));
            return $result;
        } catch (ApiException $e) {
            $error = \Tools::jsonEncode(array('message' => $e->getResponseBody()));
            zipMoneyLogger::error("Error:-" . $e->getCode() . "-" . json_encode($e->getResponseBody()));
            if (
                $e->getCode() == 402 &&
                $mapped_error_code = ErrorHandling::getMappedErrorCode($e->getResponseObject()->getError()->getCode())
            ) {
                if ($mapped_error_code) {
                    $this->context->smarty->assign('ps_error_code', $mapped_error_code);
                    return array('zm_error' => $mapped_error_code);
                }
            }
            return 'error';
        } catch (\Exception $e) {
            $error = \Tools::jsonEncode(array('message' => $e->getMessage()));
            zipMoneyLogger::error("Payment Exception: " . $error);
        }
    }

    /**
     * Retrieves a charge. Makes a request to /charges/{id} endpoint. Returns a Charge Object when successful.
     *
     * @access public
     * @param  string $charge_id
     * @return zipMoney\Response object
     */
    public function retrieveCharge($charge_id)
    {
        \zipMoney\Configuration::getDefaultConfiguration()->setApiKey('Authorization', "Bearer " . $this->getMerchantKey());
        \zipMoney\Configuration::getDefaultConfiguration()->setEnvironment($this->getEnvironment());
        \zipMoney\Configuration::getDefaultConfiguration()->setPlatform($this->getPlatform());

        $chargeApi = Sdk::getApi(Sdk::CHARGE_API_CLASS);
        try {
            $result = $chargeApi->chargesRetrieve($charge_id);
            return $result;
        } catch (ApiException $e) {
            $error = \Tools::jsonEncode(array('message' => $e->getResponseBody()));
            zipMoneyLogger::error("Payment Api Exception: " . $error);
        } catch (\Exception $e) {
            $error = \Tools::jsonEncode(array('message' => $e->getMessage()));
            zipMoneyLogger::error("Payment Exception: " . $error);
        }
        return false;
    }

    /**
     * Prepares $requestArray array for refund
     *
     * @access public
     * @param  object $order, object $request, float $refund_amount, string $refund_reason (optional)
     * @return object $request
     */
    protected function _prepareRequestForRefund($order, $refund_amount, $refund_reason = "No reason provided")
    {
        $refund = new \zipMoney\Model\CreateRefundRequest();

        $charge_id = null;
        $payments = $order->getOrderPaymentCollection();
        foreach ($payments as $key => $payment) {
            $charge_id = $payment->transaction_id;
        }

        if ($charge_id) {
            $refund->setChargeId($charge_id);
            $refund->setReason($refund_reason);
            $refund->setAmount($refund_amount);

            return $refund;
        }
        return false;
    }

    /**
     * Prepares $requestArray array for checkout
     *
     * @access public
     * @param  object $order, object $request
     * @return object $request
     */
    protected function _prepareRequestForCheckout($cart)
    {
        $request = new \zipMoney\Model\CreateCheckoutRequest();
        $request->setShopper($this->_setShopperInfo($cart));
        $request->setOrder($this->_setOrder($cart));
        $request->setConfig($this->_setConfiguration());
        $request->setMetadata($this->_setMetaData());

        return $request;
    }
}
